"""Package provides API for testing output with emoji."""

__version__: str = "0.2.0"
__package_name__: str = "pytest-emoji-output"
__author__: str = "Volodymyr Yahello"
__email__: str = "vyahello@gmail.com"
__license__: str = "MIT"
